# SPDX-License-Identifier: Apache-2.0
#
# The OpenSearch Contributors require contributions made to
# this file be licensed under the Apache-2.0 license or a
# compatible open source license.
#
# Modifications Copyright OpenSearch Contributors. See
# GitHub history for details.
from typing import Any, Union

from ..client.utils import NamespacedClient as NamespacedClient

class AlertingClient(NamespacedClient):
    def search_monitor(
        self, body: Any, params: Union[Any, None] = ..., headers: Union[Any, None] = ...
    ) -> Union[bool, Any]: ...
    def get_monitor(
        self,
        monitor_id: Any,
        params: Union[Any, None] = ...,
        headers: Union[Any, None] = ...,
    ) -> Union[bool, Any]: ...
    def run_monitor(
        self,
        monitor_id: Any,
        params: Union[Any, None] = ...,
        headers: Union[Any, None] = ...,
    ) -> Union[bool, Any]: ...
    def create_monitor(
        self,
        body: Union[Any, None] = ...,
        params: Union[Any, None] = ...,
        headers: Union[Any, None] = ...,
    ) -> Union[bool, Any]: ...
    def update_monitor(
        self,
        monitor_id: Any,
        body: Union[Any, None] = ...,
        params: Union[Any, None] = ...,
        headers: Union[Any, None] = ...,
    ) -> Union[bool, Any]: ...
    def delete_monitor(
        self,
        monitor_id: Any,
        params: Union[Any, None] = ...,
        headers: Union[Any, None] = ...,
    ) -> Union[bool, Any]: ...
    def get_destination(
        self,
        destination_id: Union[Any, None] = ...,
        params: Union[Any, None] = ...,
        headers: Union[Any, None] = ...,
    ) -> Union[bool, Any]: ...
    def create_destination(
        self,
        body: Union[Any, None] = ...,
        params: Union[Any, None] = ...,
        headers: Union[Any, None] = ...,
    ) -> Union[bool, Any]: ...
    def update_destination(
        self,
        destination_id: Any,
        body: Union[Any, None] = ...,
        params: Union[Any, None] = ...,
        headers: Union[Any, None] = ...,
    ) -> Union[bool, Any]: ...
    def delete_destination(
        self,
        destination_id: Any,
        params: Union[Any, None] = ...,
        headers: Union[Any, None] = ...,
    ) -> Union[bool, Any]: ...
    def get_alerts(
        self, params: Union[Any, None] = ..., headers: Union[Any, None] = ...
    ) -> Union[bool, Any]: ...
    def acknowledge_alert(
        self,
        monitor_id: Any,
        body: Union[Any, None] = ...,
        params: Union[Any, None] = ...,
        headers: Union[Any, None] = ...,
    ) -> Union[bool, Any]: ...



# SPDX-License-Identifier: Apache-2.0
#
# The OpenSearch Contributors require contributions made to
# this file be licensed under the Apache-2.0 license or a
# compatible open source license.
#
# Modifications Copyright OpenSearch Contributors. See
# GitHub history for details.

from typing import Any, Dict, List

class AWSV4SignerAsyncAuth:
    @property
    def __init__(self, *args: Any, **kwargs: Any) -> None: ...
    @property
    def __call__(self, *args: Any, **kwargs: Any) -> Any: ...
    @property
    def _sign_request(self, *args: Any, **kwargs: Any) -> Dict[str, List[str]]: ...



# SPDX-License-Identifier: Apache-2.0
#
# The OpenSearch Contributors require contributions made to
# this file be licensed under the Apache-2.0 license or a
# compatible open source license.
#
# Modifications Copyright OpenSearch Contributors. See
# GitHub history for details.

import sys

PY3 = sys.version_info[0] == 3


class AWSV4SignerAsyncAuth:
    """
    AWS V4 Request Signer for Async Requests.
    """

    def __init__(self, credentials, region, service="es"):  # type: ignore
        if not credentials:
            raise ValueError("Credentials cannot be empty")
        self.credentials = credentials

        if not region:
            raise ValueError("Region cannot be empty")
        self.region = region

        if not service:
            raise ValueError("Service name cannot be empty")
        self.service = service

    def __call__(self, method, url, query_string, body):  # type: ignore
        return self._sign_request(method, url, query_string, body)  # type: ignore

    def _sign_request(self, method, url, query_string, body):
        """
        This method helps in signing the request by injecting the required headers.
        :param prepared_request: unsigned headers
        :return: signed headers
        """

        from botocore.auth import SigV4Auth
        from botocore.awsrequest import AWSRequest

        # create an AWS request object and sign it using SigV4Auth
        aws_request = AWSRequest(
            method=method,
            url="".join([url, query_string]),
        )

        sig_v4_auth = SigV4Auth(self.credentials, self.service, self.region)
        sig_v4_auth.add_auth(aws_request)

        # copy the headers from AWS request object into the prepared_request
        return dict(aws_request.headers.items())



# SPDX-License-Identifier: Apache-2.0
#
# The OpenSearch Contributors require contributions made to
# this file be licensed under the Apache-2.0 license or a
# compatible open source license.
#
# Modifications Copyright OpenSearch Contributors. See
# GitHub history for details.

from typing import Any, Mapping, Optional

from .._async._extra_imports import aiohttp  # type: ignore
from .._async.http_aiohttp import AIOHttpConnection

class AsyncHttpConnection(AIOHttpConnection):
    session: Optional[aiohttp.ClientSession]
    def __init__(
        self,
        host: str = ...,
        port: Optional[int] = ...,
        http_auth: Optional[Any] = ...,
        use_ssl: bool = ...,
        verify_certs: bool = ...,
        ssl_show_warn: bool = ...,
        ca_certs: Optional[Any] = ...,
        client_cert: Optional[Any] = ...,
        client_key: Optional[Any] = ...,
        ssl_version: Optional[Any] = ...,
        ssl_assert_fingerprint: Optional[Any] = ...,
        maxsize: Optional[int] = ...,
        headers: Optional[Mapping[str, str]] = ...,
        ssl_context: Optional[Any] = ...,
        http_compress: Optional[bool] = ...,
        opaque_id: Optional[str] = ...,
        loop: Optional[Any] = ...,
        **kwargs: Any
    ) -> None: ...



# SPDX-License-Identifier: Apache-2.0
#
# The OpenSearch Contributors require contributions made to
# this file be licensed under the Apache-2.0 license or a
# compatible open source license.
#
# Modifications Copyright OpenSearch Contributors. See
# GitHub history for details.

import sys

import requests

PY3 = sys.version_info[0] == 3

if PY3:
    from urllib.parse import parse_qs, urlencode, urlparse


def fetch_url(prepared_request):  # type: ignore
    """
    This is a util method that helps in reconstructing the request url.
    :param prepared_request: unsigned request
    :return: reconstructed url
    """
    url = urlparse(prepared_request.url)
    path = url.path or "/"

    # fetch the query string if present in the request
    querystring = ""
    if url.query:
        querystring = "?" + urlencode(
            parse_qs(url.query, keep_blank_values=True), doseq=True
        )

    # fetch the host information from headers
    headers = dict(
        (key.lower(), value) for key, value in prepared_request.headers.items()
    )
    location = headers.get("host") or url.netloc

    # construct the url and return
    return url.scheme + "://" + location + path + querystring


class AWSV4SignerAuth(requests.auth.AuthBase):
    """
    AWS V4 Request Signer for Requests.
    """

    def __init__(self, credentials, region, service="es"):  # type: ignore
        if not credentials:
            raise ValueError("Credentials cannot be empty")
        self.credentials = credentials

        if not region:
            raise ValueError("Region cannot be empty")
        self.region = region

        if not service:
            raise ValueError("Service name cannot be empty")
        self.service = service

    def __call__(self, request):  # type: ignore
        return self._sign_request(request)  # type: ignore

    def _sign_request(self, prepared_request):  # type: ignore
        """
        This method helps in signing the request by injecting the required headers.
        :param prepared_request: unsigned request
        :return: signed request
        """

        from botocore.auth import SigV4Auth
        from botocore.awsrequest import AWSRequest

        url = fetch_url(prepared_request)  # type: ignore

        # create an AWS request object and sign it using SigV4Auth
        aws_request = AWSRequest(
            method=prepared_request.method.upper(),
            url=url,
        )

        sig_v4_auth = SigV4Auth(self.credentials, self.service, self.region)
        sig_v4_auth.add_auth(aws_request)

        # copy the headers from AWS request object into the prepared_request
        prepared_request.headers.update(dict(aws_request.headers.items()))

        return prepared_request



# SPDX-License-Identifier: Apache-2.0
#
# The OpenSearch Contributors require contributions made to
# this file be licensed under the Apache-2.0 license or a
# compatible open source license.
#
# Modifications Copyright OpenSearch Contributors. See
# GitHub history for details.
#

from typing import Any, Union

from ..client.utils import NamespacedClient as NamespacedClient
from ..client.utils import query_params as query_params

class AlertingClient(NamespacedClient):
    def search_monitor(
        self, body: Any, params: Any | None = ..., headers: Any | None = ...
    ) -> Union[bool, Any]: ...
    def get_monitor(
        self, monitor_id: Any, params: Any | None = ..., headers: Any | None = ...
    ) -> Union[bool, Any]: ...
    def run_monitor(
        self, monitor_id: Any, params: Any | None = ..., headers: Any | None = ...
    ) -> Union[bool, Any]: ...
    def create_monitor(
        self,
        body: Any | None = ...,
        params: Any | None = ...,
        headers: Any | None = ...,
    ) -> Union[bool, Any]: ...
    def update_monitor(
        self,
        monitor_id: Any,
        body: Any | None = ...,
        params: Any | None = ...,
        headers: Any | None = ...,
    ) -> Union[bool, Any]: ...
    def delete_monitor(
        self, monitor_id: Any, params: Any | None = ..., headers: Any | None = ...
    ) -> Union[bool, Any]: ...
    def get_destination(
        self,
        destination_id: Any | None = ...,
        params: Any | None = ...,
        headers: Any | None = ...,
    ) -> Union[bool, Any]: ...
    def create_destination(
        self,
        body: Any | None = ...,
        params: Any | None = ...,
        headers: Any | None = ...,
    ) -> Union[bool, Any]: ...
    def update_destination(
        self,
        destination_id: Any,
        body: Any | None = ...,
        params: Any | None = ...,
        headers: Any | None = ...,
    ) -> Union[bool, Any]: ...
    def delete_destination(
        self, destination_id: Any, params: Any | None = ..., headers: Any | None = ...
    ) -> Union[bool, Any]: ...
    def get_alerts(
        self, params: Any | None = ..., headers: Any | None = ...
    ) -> Union[bool, Any]: ...
    def acknowledge_alert(
        self,
        monitor_id: Any,
        body: Any | None = ...,
        params: Any | None = ...,
        headers: Any | None = ...,
    ) -> Union[bool, Any]: ...
